﻿local AceAddon = AceLibrary("AceAddon-2.0")
local bs = AceLibrary("Babble-Spell-2.2")
local L = AceLibrary("AceLocale-2.2"):new("Cartographer-Noteshare")
local Tourist = AceLibrary("Tourist-2.0")

L:RegisterTranslations("enUS", function() return {
	["Database sharing"] = "Database sharing",
	["Module Description"] = "Module to share your notes automatically.",
	["Guild"] = true,
	["Group"] = true,
	["General"] = true,
	["Sharing Channel"] = true,
	["What channel you broadcast notes on."] = true,
	["Shared Databases"] = true,
	["Databases to share automatically"] = true,
	["Treasure"] = true,
	["Broadcast Database"] = true,
	["Broadcast your database on your selected channel."] = true,
	["Receive filter"] = true,
	["Only accept broadcast notes for these databases."] = true,	
} end)  

L:RegisterTranslations("deDE", function() return {
	["Database sharing"] = "Datenbank-Abgleich",
	["Module Description"] = "Notizen automatisch mit anderen abgleichen.",
	["Guild"] = "Gilde",
	["Group"] = "Gruppe",
	["General"] = "Allgemein",
	["Sharing Channel"] = "Kanal",
	["What channel you broadcast notes on."] = "\195\156ber diesen Kanal werden die Notizen gesendet.",
	["Shared Databases"] = "Datenbanken",
	["Databases to share automatically"] = "Datenbanken, die automatisch mit anderen abgeglichen werden.",
	["Treasure"] = "Sch\195\164tze",
	["Broadcast Database"] = "Datenbank senden",
	["Broadcast your database on your selected channel."] = "Gesamte Datenbank an den eingestellten Kanal senden.",
	["Receive filter"] = "Eingangsfilter",
	["Only accept broadcast notes for these databases."] = "Nur eingehende Notizen f\195\188r diese Datenbanken akzeptieren.",	
} end)  

L:RegisterTranslations("koKR", function() return {
	["Database sharing"] = "데이터베이스 공유",
	["Module Description"] = "당신의 노트를 자동으로 공유하는 모듈입니다.",
	["Guild"] = "길드",
	["Group"] = "그룹",
	["Sharing Channel"] = "채널 공유",
	["What channel you broadcast notes on."] = "당신의 노트를 채널에 공유합니다.",
	["Shared Databases"] = "공유 데이터베이스",
	["Databases to share automatically"] = "자동적으로 데이터베이스를 공유합니다.",
	["Treasure"] = "보물",
	["Broadcast Database"] = "데이터베이스 알림",
	["Broadcast your database on your selected channel."] = "당신이 선택한 채널에 당신의 데이버베이스를 알림니다.",
	["Receive filter"] = "필터 받기",
	["Only accept broadcast notes for these databases."] = "데이터베이스를 위해 노트 알림을 받습니다.",
} end)  

local Cartographer_Noteshare = Cartographer:NewModule("Noteshare","AceEvent-2.0", "AceConsole-2.0","AceHook-2.1")
local cartNoteHook = nil
local hasInstanceMaps = false

local dblist = {
	["Mining"] = bs["Mining"],
	["Herbalism"] = bs["Herbalism"],
	["Treasure"] = L["Treasure"],
	["POI"] = AceLibrary("AceLocale-2.2"):new("Cartographer_POI")["POI"],
	["General"] = L["General"],
	["Fishing"] = bs["Fishing"],
}
table.sort(dblist)

function Cartographer_Noteshare:OnInitialize()
	self.db = Cartographer:AcquireDBNamespace("Noteshare")
	Cartographer:RegisterDefaults("Noteshare", "profile", {
		broadcastChannel = "GUILD",
		shareDB = {},
		filterDB = {},
		shareProf = false,
		shareInst = false,
		sharePOI = false,
	})
	self.title   = L["Database sharing"]
	self.name    = L["Database sharing"]
	self.author  = "Kagaro on Ysera"
	self.notes   = L["Module Description"]
	self.email   = "sal.scotto@gmail.com"
	self.version = "1.0"
	self.website = nil
	if Cartographer_InstanceMaps then
		hasInstanceMaps = true
	end
	Cartographer.options.args.Noteshare = {
		name = L["Database sharing"],
		desc = self.notes,
		handler = self,
		type = 'group',
		args = {
			toggle = {
				name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
				desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
				type  = "toggle",
				order = -1,
				get   = function() return Cartographer:IsModuleActive(self) end,
				set   = function() Cartographer:ToggleModuleActive(self) end,				
			},
			-- share channel
			channel = {
				name = L["Sharing Channel"],
				desc = L["What channel you broadcast notes on."],
				type = "text",
				order = 1,
				get = "GetBroadcastChannel",
				set = "SetBroadcastChannel",
				validate = {["GUILD"] = L["Guild"], ["GROUP"] = L["Group"]},
				disabled = function() return not Cartographer:IsModuleActive(self) end,
			},
			-- databases to share
			shared_db = {
				name = L["Shared Databases"],
				desc = L["Databases to share automatically"],
				type = "group",
				order = 2,
				args = {},
				disabled = function() return not Cartographer:IsModuleActive(self) end
			},
			dump_database = {
				name = L["Broadcast Database"],
				desc = L["Broadcast your database on your selected channel."],
				type = "group",
				order = 4,
				args = {},
				disabled = function() return not Cartographer:IsModuleActive(self) end,
			},
			ignored_databases = {
				name = L["Receive filter"],
				desc = L["Only accept broadcast notes for these databases."],
				type = "group",
				order = 3,
				args = {},
				disabled = function() return not Cartographer:IsModuleActive(self) end,
			}
		},
	}
	-- setup sub groups
	for k,v in pairs(dblist) do
		local k = k
		Cartographer.options.args.Noteshare.args.shared_db.args[k] = {
			name = v,
			desc = v,
			type = 'toggle',
			get = function() return self:IsSharingDatabase(k) end,
			set = function() return self:ToggleSharingDatabase(k) end,
		}
		Cartographer.options.args.Noteshare.args.dump_database.args[k] = {
			name = v,
			desc = v,
			type = 'execute',
			func = function() self:DumpDatabase(k) end,
		}
		Cartographer.options.args.Noteshare.args.ignored_databases.args[k] = {
			name = v,
			desc = v,
			type = 'toggle',
			get = function() return self:IsFilteringDatabase(k) end,
			set = function() return self:ToggleFilteringDatabase(k) end,			
		}
	end
	AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(self, Cartographer.options.args.Noteshare)
	
end

function Cartographer_Noteshare:OnEnable()
	if Cartographer_Notes then
		-- register our listener
		self:StartEvents()
	else
		-- if the Notes module not active we dont do anything
		Cartographer:ToggleModuleActive(self,false)
	end
end

function Cartographer_Noteshare:OnDisable()
	self:StopEvents()
end

function Cartographer_Noteshare:StartEvents()
	self:RegisterEvent("CartographerNotes_NoteSet", function(zone,x,y,icon,creator,comm) 
		if not comm then 
			self:NoteAdded(zone,x,y,icon,creator)
		end 
	end)
	-- hook Cartographer.OnCommReceive
	self:Hook(Cartographer_Notes,"OnCommReceive","HookedComm")
end

function Cartographer_Noteshare:HookedComm(cart_note_ref, prefix, sender, distribution, cart_note_table, zone, x, y, creator, data)
	-- ok we hook on comm receive lets filter
	if type(zone) ~= "string" or type(x) ~= "number" or x < 0 or x > 1 or type(y) ~= "number" or y < 0 or y > 1 or (type(creator) ~= "string" and creator ~= nil) or (type(data) ~= "table" and type(data) ~= "string") or (type(data) == "table" and type(data.icon) ~= "string") then
		-- bad data
		return
	end
	
	-- well say any not coming that not one of the registered databases is a general note
	local isRegistered = (Cartographer_Notes.externalDBs[creator] and creator) or (dblist[creator] and creator)
-- I could just use the cart_note_ref but ill use the proper hook method
	if self.db.profile.filterDB[creator] or (self.db.profile.filterDB["General"] and not isRegistered) then
		self.hooks[Cartographer_Notes]["OnCommReceive"].NOTE(Cartographer_Notes,prefix,sender,distribution,zone,x,y,creator,data)
	end
end

function Cartographer_Noteshare:StopEvents()
	self:UnregisterAllEvents()
	self:UnhookAll()
end

-- Prefeneces setup
function Cartographer_Noteshare:GetBroadcastChannel()
	return self.db.profile.broadcastChannel
end
function Cartographer_Noteshare:SetBroadcastChannel(value)
	self.db.profile.broadcastChannel = value
end

function Cartographer_Noteshare:ToggleSharingDatabase(v)
	self.db.profile.shareDB[v] = not self.db.profile.shareDB[v]
end

function Cartographer_Noteshare:IsSharingDatabase(v)
	if self.db.profile.shareDB[v] == nil then
		self.db.profile.shareDB[v] = true
	end
	return self.db.profile.shareDB[v]
end

function Cartographer_Noteshare:ToggleFilteringDatabase(v)
	self.db.profile.filterDB[v] = not self.db.profile.filterDB[v]
end
function Cartographer_Noteshare:IsFilteringDatabase(v)
	if self.db.profile.filterDB[v] == nil then
		self.db.profile.filterDB[v] = true
	end
	return self.db.profile.filterDB[v]
end

function Cartographer_Noteshare:DumpDatabase(v)
	local zone = Cartographer:GetCurrentEnglishZoneName()
	if v == "General" then
		for zone,x,y,icon,creator,data in Cartographer_Notes:IterateNearbyNotes(zone,1,1,10000,nil,nil,true) do
			if not dblist[creator] or not Cartographer_Notes.externalDBs[creator] then
					self:BroadcastNote(zone,x,y)
			end
		end
	else
		-- ocheck that we have this db before trying to send it
		if Cartographer_Notes.externalDBs[v] then
			for zone,x,y,icon,creator,data in Cartographer_Notes:IterateNearbyNotes(zone,1,1,10000,v,nil,true) do
				self:BroadcastNote(zone,x,y)
			end
		end
	end
end

-- fire the note of on our broadcast channel
function Cartographer_Noteshare:BroadcastNote(zone,x,y)
	if self.db.profile.broadcastChannel == "GROUP" then
		Cartographer_Notes:SendNoteToGroup(zone,x,y)
	elseif self.db.profile.broadcastChannel == "GUILD" then
		Cartographer_Notes:SendNoteToGuild(zone,x,y)
	end
end

function Cartographer_Noteshare:IsNoteManual(creator)
	return not (creator and Cartographer_Notes.externalDBs[creator])
end

-- note add callback
function Cartographer_Noteshare:NoteAdded(zone,x,y,icon,creator)
	if self.db.profile.shareDB[creator] then
		self:BroadcastNote(zone,x,y)
	elseif self.db.profile.shareDB["General"] and self:IsNoteManual(creator) then
		self:BroadcastNote(zone,x,y)	
	end
end

                                         